@@ -28,6 +28,7 @@ Tool to check all requirements are actually required.
 
 # stdlib
 import ast
+import re
 from collections import defaultdict
 from typing import Any, Dict, List, Optional, Tuple
 
@@ -40,12 +41,12 @@ from shippinglabel.requirements import read_requirements
 from stdlib_list import stdlib_list
 
 # this package
-from dep_checker.config import AllowedUnused, ConfigReader, NameMapping
+from dep_checker.config import AllowedUnused, ConfigReader, NameMapping, NamespacePackages
 
 __author__: str = "Dominic Davis-Foster"
 __copyright__: str = "2020 Dominic Davis-Foster"
 __license__: str = "MIT License"
-__version__: str = "0.2.0"
+__version__: str = "0.3.0"
 __email__: str = "dominic@davis-foster.co.uk"
 
 __all__ = ["template", "check_imports"]
@@ -58,24 +59,31 @@ template = "{name} imported on line {lineno} of {filename}"
 
 class Visitor(ast.NodeVisitor):
 
-	def __init__(self, pkg_name: str):
+	def __init__(self, pkg_name: str, namespace_packages: Optional[Dict[str, List[str]]] = None):
 		self.import_sources = []
 		self.pkg_name = pkg_name
+		self.namespace_packages = namespace_packages or {}
 
 	def record_import(self, name: str, lineno: int):
-		name = name.split(".")[0]
+		# TODO: handle ``from namespace import package``
+
+		for namespace in self.namespace_packages:
+			if namespace in name:
+				name = name.partition(namespace)[1].replace(".", "_")
+				break
+		else:
+			# Not a namespace package
+			name = name.split(".")[0]
 
 		if name not in libraries and name != self.pkg_name:
 			self.import_sources.append((name, lineno))
 
 	def visit_Import(self, node: ast.Import) -> None:
-		# TODO: allow ignoring unused with "# noqa, or maybe "# nodep"?"
 		for name in node.names:
 			name: ast.alias
 			self.record_import(name.name, node.lineno)
 
 	def visit_ImportFrom(self, node: ast.ImportFrom) -> None:
-		# TODO: allow ignoring unused with "# noqa, or maybe "# nodep"?"
 		if node.level != 0:
 			# relative import
 			return
@@ -140,6 +148,7 @@ def check_imports(
 		allowed_unused: Optional[List[str]] = None,
 		colour: Optional[bool] = None,
 		name_mapping: Optional[Dict[str, str]] = None,
+		namespace_packages: Optional[List[str]] = None,
 		) -> int:
 	"""
 	Check imports for the given package, against the given requirements file.
@@ -156,13 +165,13 @@ def check_imports(
 	| Returns ``0`` if all requirements are used and listed as requirements.
 	| Returns ``1`` is a requirement is unused, or if a package is imported but not listed as a requirement.
 
-	.. versionchanged:: 0.2.0 Added the ``name_mapping`` option.
+	.. versionchanged:: 0.3.0 Added the ``name_mapping`` option.
 	"""
 
-	colour = resolve_color_default(colour)
-
+	ret = 0
 	cwd = PathPlus.cwd()
 	config = reader.visit()
+	colour = resolve_color_default(colour)
 
 	if allowed_unused is None:
 		allowed_unused = AllowedUnused.get(config)
@@ -170,12 +179,26 @@ def check_imports(
 	if name_mapping is None:
 		name_mapping = NameMapping.get(config)
 
+	if namespace_packages is None:
+		namespace_packages = NamespacePackages.get(config)
+
 	req_file = PathPlus(req_file)
 
 	if not req_file.is_absolute():
 		req_file = cwd / req_file
 
-	req_names = sorted(req.name.replace("-", "_") for req in read_requirements(req_file)[0])
+	req_names = []
+
+	for req in read_requirements(req_file)[0]:
+		name = req.name.replace("-", "_")
+		if name in name_mapping:
+			# replace names in req_names with the name of the package the requirement provides
+			req_names.append(name_mapping[name])
+		else:
+			req_names.append(name)
+
+	req_names.sort()
+
 	imports: Dict[str, Dict[PathPlus, int]] = defaultdict(dict)
 	# mapping of import name to mapping of filename to lineno where imported
 
@@ -183,35 +206,40 @@ def check_imports(
 		raise FileNotFoundError(f"Can't find a package called {pkg_name} in the current directory.")
 
 	for filename in (cwd / pkg_name).rglob("*.py"):
-		if filename.relative_to(cwd).parts[0] in {".tox", "venv", ".venv"}:
+		filename = filename.relative_to(cwd)
+
+		if filename.parts[0] in {".tox", "venv", ".venv"}:
 			continue
 
-		visitor = Visitor(pkg_name)
-		for name, lineno in visitor.visit(ast.parse(filename.read_text())):
-			imports[name][filename.relative_to(cwd)] = min(
-					(imports[name].get(filename.relative_to(cwd), lineno), lineno)
-					)
+		visitor = Visitor(pkg_name, namespace_packages)
+		file_content = filename.read_text()
 
-	ret = 0
+		for import_name, lineno in visitor.visit(ast.parse(file_content)):
 
-	# replace names in req_names with the name of the package the requirement provides
-	req_names = [name if name not in name_mapping else name_mapping[name] for name in req_names]
+			if import_name not in req_names:
+				# Not listed as requirement
+
+				if re.match(r".*#\s*nodep.*", file_content.splitlines()[lineno - 1]):
+					# Marked with "# nodep", so the user wants to ignore this
+					continue
+
+				msg = template.format(name=import_name, lineno=lineno, filename=filename)
+				click.echo(Fore.RED(f"✘ {msg} but not listed as a requirement"), color=colour)
+				ret |= 1
+
+			else:
+				imports[import_name][filename] = min((imports[import_name].get(filename, lineno), lineno))
 
 	for req in req_names:
 		for filename, lineno in imports[req].items():
+			# Imported and listed as requirement
 			msg = template.format(name=req, lineno=lineno, filename=filename)
 			click.echo(Fore.GREEN(f"✔ {msg}"), color=colour)
 			break
 		else:
 			if req not in allowed_unused:
+				# not imported
 				click.echo(Fore.YELLOW(f"✘ {req} never imported"), color=colour)
 				ret |= 1
 
-	for import_name, locations in imports.items():
-		if import_name not in req_names:
-			for filename, lineno in locations.items():
-				msg = template.format(name=import_name, lineno=lineno, filename=filename)
-				click.echo(Fore.RED(f"✘ {msg} but not listed as a requirement"), color=colour)
-				ret |= 1
-
 	return ret
